@@ -140,9 +140,9 @@ int mqmaugmentfull(MQMMarkerMatrix* markers,int* nind, int* augmentednind, ivect
     if (verbose) Rprintf("INFO: Augmentation routine\n");
     const int nind0 = *nind;
     const vector originalpheno = (*pheno_value)[0];
-    MQMMarkerMatrix newmarkerset;
-    vector new_y;                   //Because we do a phenotype matrix, we optimize by storing original the R-individual 
-    ivector new_ind;                //numbers inside the trait-values, ands use new_ind etc for inside C
+    MQMMarkerMatrix newmarkerset;   // [Danny:] This LEAKS MEMORY the Matrices and vectors are not cleaned at ALL
+    vector new_y;                   // Because we do a phenotype matrix, we optimize by storing original the R-individual 
+    ivector new_ind;                // numbers inside the trait-values, ands use new_ind etc for inside C
     ivector succes_ind;
     cvector position = relative_marker_position(nmark,chr);
     vector r = recombination_frequencies(nmark, position, mapdistance);
@@ -238,6 +238,10 @@ int mqmaugmentfull(MQMMarkerMatrix* markers,int* nind, int* augmentednind, ivect
       (*augmentednind)=(*augmentednind)+(numimputations*current_leftover_ind);
       (*nind)= (*nind)+(current_leftover_ind);
       debug_trace("nind:%d,naugmented:%d",(*nind)+(current_leftover_ind),(*augmentednind)+(current_leftover_ind));
+      Rprintf("INFO: VALGRIND MEMORY DEBUG BARRIERE TRIGGERED\n", "");
+      delMQMMarkerMatrix(newmarkerset, nmark);    // Free the newmarkerset, this can only be done here since: (*markers) = newmarkerset_all;
+      // Free(new_y_all);
+      // Free(new_ind_all);
     }else{
       if(ind_still_left && augment_strategy == 3){
         if(verbose) Rprintf("INFO: Dropping %d augment_strategy individuals from further analysis\n",ind_still_left);
@@ -248,6 +252,11 @@ int mqmaugmentfull(MQMMarkerMatrix* markers,int* nind, int* augmentednind, ivect
       (*markers) = newmarkerset;
     }
     if(verbose) Rprintf("INFO: Done with augmentation\n");
+    // Free(new_y);                                // Free vector indicating the new phenotypes
+    // Free(new_ind);                              // Free vector indicating the new individuals
+    Free(succes_ind);                           // Free vector indicating the result of round 1 - augmentation
+    Free(position);                             // Free the positions of the markers
+    Free(r);                                    // Free the recombination frequencies
     return 1;
 }
 
@@ -621,7 +630,7 @@ bailout:
   retvalue = 0;
 cleanup:
   Free(newy);
-  Free(newmarker);
+  delMQMMarkerMatrix(newmarker, Nmark+1); //Free(newmarker);
   Free(newind);
   Free(newprob);
   Free(newprobmax);
@@ -650,50 +659,35 @@ void R_mqmaugment(int *geno, double *dist, double *pheno, int *auggeno,
 
   if(verbose) Rprintf("INFO: Starting C-part of the data augmentation routine\n");
   ivector new_ind;
-  vector mapdistance;
-  cvector position;
-  MQMMarkerMatrix markers, new_markers;
-  ivector chr;
 
-  markers= newMQMMarkerMatrix(*Nmark, nind0);
-  new_markers= newMQMMarkerMatrix(*Nmark, *maxind);
-  mapdistance = newvector(*Nmark);
-  chr= newivector(*Nmark);
+  MQMMarkerMatrix markers = newMQMMarkerMatrix(*Nmark, nind0);
+  vector mapdistance = newvector(*Nmark);
+  ivector chr = newivector(*Nmark);
 
   //Reorganise the pointers into arrays, Singletons are just cast into the function
   reorg_geno(nind0, *Nmark, geno, &Geno);
   reorg_int(*Nmark, 1, chromo, &Chromo);
   reorg_pheno(nind0, *Npheno, pheno, &Pheno);
   reorg_pheno(*Nmark, 1, dist, &Dist);
-
   reorg_int(*maxind, *Nmark, auggeno, &NEW);
   reorg_int((*maxiaug)*nind0, 1, augIND, &NEWIND);
   reorg_pheno((*maxiaug)*nind0, 1, augPheno, &NEWPheno);
 
-  MQMCrossType crosstype = determine_MQMCross(*Nmark, *Nind, (const int **)Geno, rqtlcrosstype);
-  //Change all the markers from R/qtl format to MQM internal
-  change_coding(Nmark, Nind, Geno, markers, crosstype);
+  MQMCrossType crosstype = determine_MQMCross(*Nmark, *Nind, (const int **)Geno, rqtlcrosstype);        // Determine cross
+  change_coding(Nmark, Nind, Geno, markers, crosstype);                                                 // Change all the markers from R/qtl format to MQM internal
 
   if(verbose) Rprintf("INFO: Filling the chromosome matrix\n");
   for (int i=0; i<(*Nmark); i++) {
     //Set some general information structures per marker
-    mapdistance[i]=POSITIONUNKNOWN;
-    mapdistance[i]=Dist[0][i];
+    mapdistance[i] = POSITIONUNKNOWN;
+    mapdistance[i] = Dist[0][i];
     chr[i] = Chromo[0][i];
   }
-  //Calculate positions of markers and Recombinant frequencies
-  position = relative_marker_position(*Nmark,chr);
-  vector r = recombination_frequencies(*Nmark, position, mapdistance);
-  //ivector succes_ind;
-  /*
-  if (mqmaugment(markers, Pheno[(*Npheno-1)], &new_markers, &new_y, &new_ind, &succes_ind, Nind, Naug, *Nmark, position, r, *maxind, *maxiaug, *minprob, crosstype, verbose)==1) {
-  
-  */
+
   if(mqmaugmentfull(&markers,Nind,Naug,&new_ind,*minprob, *maxind, *maxiaug,&Pheno,*Nmark,chr,mapdistance,*augment_strategy,crosstype,verbose)){
-    //Data augmentation finished succesfully
-    //Push it back into RQTL format
-    for (int i=0; i<(*Nmark); i++) {
-      for (int j=0; j<(*Naug); j++) {
+    //Data augmentation finished succesfully, encode it back into RQTL format
+    for (int i = 0; i<(*Nmark); i++) {
+      for (int j = 0; j<(*Naug); j++) {
         //Rprintf("INFO: Phenotype after return: %f",NEWPheno[0][j]);
         NEWPheno[0][j] = Pheno[0][j];
         NEWIND[0][j] = new_ind[j];
@@ -715,18 +709,12 @@ void R_mqmaugment(int *geno, double *dist, double *pheno, int *auggeno,
         }
       }
     }
-    //delMQMMarkerMatrix(new_markers,*Nmark);
-    //delMQMMarkerMatrix(markers,*Nmark);
-    Free(mapdistance);
-    Free(position);
-    Free(r);
-    Free(chr);
     if (verbose) {
       Rprintf("# Unique individuals before augmentation:%d\n", nind0);
       Rprintf("# Unique selected individuals:%d\n", *Nind);
       Rprintf("# Marker p individual:%d\n", *Nmark);
       Rprintf("# Individuals after augmentation:%d\n", *Naug);
-      Rprintf("INFO: Data augmentation succesfull");
+      Rprintf("INFO: Data augmentation succesfull\n");
     }
   } else {
     //Unsuccessfull data augmentation exit
@@ -742,8 +730,8 @@ void R_mqmaugment(int *geno, double *dist, double *pheno, int *auggeno,
         if (markers[i][j] == MH) {
           NEW[i][j] = 2;
         }
-        if (markers[i][j] == MBB) { // [karl:] this might need to be changed for RIL
-          crosstype==CRIL ? NEW[i][j]=2 : NEW[i][j] = 3;  //[Danny:] This should solve it 
+        if (markers[i][j] == MBB) {                       // [Karl:] this might need to be changed for RIL
+          crosstype==CRIL ? NEW[i][j]=2 : NEW[i][j] = 3;  // [Danny:] This should solve it 
         }
         if (markers[i][j] == MNOTAA) {
           NEW[i][j] = 5;
@@ -753,14 +741,11 @@ void R_mqmaugment(int *geno, double *dist, double *pheno, int *auggeno,
         }
       }
     }
-    delMQMMarkerMatrix(new_markers,*Nmark);
-    delMQMMarkerMatrix(markers,*Nmark);
-    Free(mapdistance);
-    Free(position);
-    Free(r);
-    Free(chr);
     fatal("Data augmentation failed", "");
   }
+  delMQMMarkerMatrix(markers,*Nmark); // [Danny:] This looked suspicious, we were leaking memory here because we didn't clean it
+  Free(mapdistance);
+  Free(chr);
   return;
 }
 
